<?php

namespace Drupal\proc\Form;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\proc\ProcInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Decrypt content.
 */
class ProcDecryptForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'proc_decrypt_form';
  }

  /**
   * Dynamic title for Decrypt form.
   */
  public function getTitle() {
    $title = $this->t('Download and decrypt');

    if (!$this->isFileInputMode()) {
      $title = $this->t('Decrypt');
    }

    return $title;
  }

  /**
   * Build the form.
   *
   * @param array $form
   *   Default form array structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object containing current form state.
   *
   * @return array
   *   The render array defining the elements of the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = \Drupal::config('proc.settings');
    $allow_password_caching = $config->get('proc-allow-password-cache-fields');

    $query_params = \Drupal::request()->query->all();

    $cache_password = NULL;
    if (isset($query_params['cache-password'])) {
      $cache_password = $query_params['cache-password'];
    }

    $form['password'] = [
      '#type' => 'password',
      '#title' => $this->t('Protected Content Password'),
      '#description' => $this->t('You must type in the password used on registering your Protected Content Key.'),
    ];

    if ($allow_password_caching && $cache_password == '1') {
      $form['cache_password'] = [
        '#type' => 'checkbox',
        '#title' => 'Cache password',
        '#attributes' => [
          // Switch value of checkbox:
          'onclick' => 'this.value = this.checked ? "1" : "0";',
        ],
      ];
    }

    if ($allow_password_caching && $cache_password == '2') {
      $form['cache_password'] = [
        '#type' => 'hidden',
        '#title' => 'Cache password',
        '#attributes' => [
          // Switch value of checkbox:
          'onclick' => 'this.value = this.checked ? "1" : "0";',
        ],
      ];
    }

    $decryp_classes = [
      'button--primary',
      'button',
    ];
    \Drupal::moduleHandler()
      ->alter('decryption_link_classes', $decryp_classes);

    $fragment = Crypt::hashBase64(Crypt::randomBytesBase64(32));

    $input_mode = \Drupal::request()->get('field-input-mode');
    $link_text = FALSE;
    if (isset($input_mode) && $input_mode !== ProcInterface::INPUT_MODE_FILE) {
      $link_text = $this->t('Decrypt');
    }

    $form['decrypt'] = [
      '#type' => 'link',
      '#title' => ($link_text) ?: $this->t('Download and decrypt'),
      '#url' => Url::fromUserInput('#' . $fragment),
      '#attributes' => [
        'id' => 'decryption-link',
        'class' => $decryp_classes,
      ],
    ];

    // Add disabled and hidden submit btn to prevent issues if Enter is pressed.
    // @todo Fix decryption form submit.
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => ($link_text) ?: $this->t('Download and decrypt'),
      '#attributes' => [
        'id' => 'decryption-btn',
        'style' => 'display: none',
        'disabled' => TRUE,
      ],
    ];

    return $form;
  }

  /**
   * Implements form validation.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {}

  /**
   * Implements a form submit handler.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {}

  /**
   * Deny access.
   */
  public function denyAccess() {
    throw new AccessDeniedHttpException();
  }

  /**
   * Validade if Proc field input mode a file field.
   */
  private function isFileInputMode() {
    $input_mode = \Drupal::request()->get('field-input-mode');
    if (!isset($input_mode)) {
      return FALSE;
    }

    return ((int) $input_mode === ProcInterface::INPUT_MODE_FILE);
  }

}
